@@ -44,6 +44,10 @@
 #include "win32/WIN32Server.h"
 #include "win32/WIN32Geometry.h"
 
+#ifdef __CYGWIN__
+#include <sys/file.h>
+#endif
+
 static void 
 validateWindow(HWND hwnd, RECT rect);
 LRESULT CALLBACK MainWndProc(HWND hwnd, UINT uMsg,
@@ -92,6 +96,23 @@ LRESULT CALLBACK MainWndProc(HWND hwnd, UINT uMsg,
 
 - (void) setupRunLoopInputSourcesForMode: (NSString*)mode
 {
+#ifdef    __CYGWIN__
+  NSRunLoop *currentRunLoop = [NSRunLoop currentRunLoop];
+  int fdMessageQueue;
+#define WIN_MSG_QUEUE_FNAME    "/dev/windows"
+
+  // Open a file descriptor for the windows message queue
+  fdMessageQueue = open (WIN_MSG_QUEUE_FNAME, O_RDONLY);
+  if (fdMessageQueue == -1)
+    {
+      NSLog(@"Failed opening %s\n", WIN_MSG_QUEUE_FNAME);
+      exit(1);
+    }
+  [currentRunLoop addEvent: (void*)fdMessageQueue
+                  type: ET_RDESC
+                  watcher: (id<RunLoopEvents>)self
+                  forMode: mode];
+#else 
   // FIXME
   NSTimer *timer;
 
@@ -101,6 +122,7 @@ LRESULT CALLBACK MainWndProc(HWND hwnd, UINT uMsg,
 		   userInfo: nil
 		   repeats: YES];
   [[NSRunLoop currentRunLoop] addTimer: timer forMode: mode];
+#endif
 }
 
 /**
@@ -143,20 +165,30 @@ LRESULT CALLBACK MainWndProc(HWND hwnd, UINT uMsg,
   [super dealloc];
 }
 
-/** Returns an instance of a class which implements the NSDraggingInfo
-    protocol. */
-- (id <NSDraggingInfo>) dragInfo
+- (void) restrictWindow: (int)win toImage: (NSImage*)image
 {
-  [self subclassResponsibility: _cmd];
-  return nil;
+  //[self subclassResponsibility: _cmd];
 }
 
-- (BOOL) slideImage: (NSImage*)image from: (NSPoint)from to: (NSPoint)to
+- (int) findWindowAt: (NSPoint)screenLocation 
+           windowRef: (int*)windowRef 
+           excluding: (int)win
 {
-  [self subclassResponsibility: _cmd];
-  return NO;
+  HWND hwnd;
+  POINT p;
+
+  p = GSScreenPointToMS(screenLocation);
+  hwnd = WindowFromPoint(p);
+  if ((int)hwnd == win)
+    {
+      hwnd = GetWindow(hwnd, GW_HWNDNEXT);
+    }
+
+  *windowRef = (int)hwnd;
+  return (int)hwnd;
 }
 
+// FIXME: The following methods wont work for multiple screens
 /* Screen information */
 - (NSSize) resolutionForScreen: (int)screen
 {
@@ -292,7 +324,7 @@ DWORD windowStyleForGSStyle(unsigned int style)
   RECT r;
   DWORD wstyle = windowStyleForGSStyle(style);
 
-  r = GSScreenRectToMS(frame);
+  r = GSScreenRectToMS(frame, style, self);
 
   NSDebugLLog(@"WTrace", @"window: %@ : %d : %d : %d", NSStringFromRect(frame),
 	      type, style, screen);
@@ -428,10 +460,10 @@ DWORD windowStyleForGSStyle(unsigned int style)
   if (op != NSWindowOut)
     {
       int flag = SW_SHOW;
+
       if (IsIconic((HWND)winNum))
         flag = SW_RESTORE;
       ShowWindow((HWND)winNum, flag); 
-      return;
     }
 
   switch (op)
@@ -444,7 +476,7 @@ DWORD windowStyleForGSStyle(unsigned int style)
       if (otherWin == 0)
 	otherWin = (int)HWND_BOTTOM;
       SetWindowPos((HWND)winNum, (HWND)otherWin, 0, 0, 0, 0, 
-		   SWP_NOSIZE | SWP_NOMOVE);
+		   SWP_NOSIZE | SWP_NOMOVE | SWP_NOACTIVATE);
       break;
     case NSWindowAbove:
       if (otherWin <= 0)
@@ -455,7 +487,7 @@ DWORD windowStyleForGSStyle(unsigned int style)
 	  winNum = (int)HWND_TOP;
 	}
       SetWindowPos((HWND) otherWin, (HWND)winNum, 0, 0, 0, 0, 
-		   SWP_NOSIZE | SWP_NOMOVE);
+		   SWP_NOSIZE | SWP_NOMOVE | SWP_NOACTIVATE);
       break;
     }
 }
@@ -469,7 +501,7 @@ DWORD windowStyleForGSStyle(unsigned int style)
   p = GSWindowOriginToMS((HWND)winNum, loc);
 
   SetWindowPos((HWND)winNum, NULL, p.x, p.y, 0, 0, 
-	       SWP_NOZORDER | SWP_NOSIZE | SWP_NOREDRAW);
+	       SWP_NOZORDER | SWP_NOSIZE);
 }
 
 - (void) placewindow: (NSRect)frame : (int) winNum
@@ -477,14 +509,15 @@ DWORD windowStyleForGSStyle(unsigned int style)
   RECT r;
   RECT r2;
   WIN_INTERN *win = (WIN_INTERN *)GetWindowLong((HWND)winNum, GWL_USERDATA);
+  NSWindow *window = GSWindowWithNumber(winNum);
 
   NSDebugLLog(@"WTrace", @"placewindow: %@ : %d", NSStringFromRect(frame), 
 	      winNum);
-  r = GSScreenRectToMS(frame);
+  r = GSScreenRectToMS(frame, [window styleMask], self);
   GetWindowRect((HWND)winNum, &r2);
 
   SetWindowPos((HWND)winNum, NULL, r.left, r.top, r.right - r.left, r.bottom - r.top, 
-	       SWP_NOZORDER | SWP_NOREDRAW);
+	       SWP_NOZORDER);
 
   if ((win->useHDC) &&
       (r.right - r.left != r2.right - r2.left) &&
@@ -521,9 +554,10 @@ DWORD windowStyleForGSStyle(unsigned int style)
 - (NSRect) windowbounds: (int) winNum
 {
   RECT r;
+  NSWindow *window = GSWindowWithNumber(winNum);
 
   GetWindowRect((HWND)winNum, &r);
-  return MSScreenRectToGS(r);
+  return MSScreenRectToGS(r, [window styleMask], self);
 }
 
 - (void) setwindowlevel: (int) level : (int) winNum
@@ -613,7 +647,7 @@ DWORD windowStyleForGSStyle(unsigned int style)
 - (void) setinputfocus: (int) winNum
 {
   NSDebugLLog(@"WTrace", @"setinputfocus: %d", winNum);
-  NSDebugLLog(@"Focus", @"Setting input focus to %d");
+  NSDebugLLog(@"Focus", @"Setting input focus to %d", winNum);
   if (winNum == 0)
     {
       NSDebugLLog(@"Focus", @" invalid focus window");
@@ -683,6 +717,18 @@ DWORD windowStyleForGSStyle(unsigned int style)
     case GSIBeamCursor:
       hCursor = LoadCursor(NULL, IDC_IBEAM);
       break;
+    case GSCrosshairCursor:
+      hCursor = LoadCursor(NULL, IDC_CROSS);
+      break;
+    case GSPointingHandCursor:
+      hCursor = LoadCursor(NULL, IDC_HAND);
+      break;
+    case GSResizeLeftRightCursor:
+      hCursor = LoadCursor(NULL, IDC_SIZEWE);
+      break;
+    case GSResizeUpDownCursor:
+      hCursor = LoadCursor(NULL, IDC_SIZENS);
+      break;
     default:
       hCursor = LoadCursor(NULL, IDC_ARROW);
       break;
