@@ -31,6 +31,7 @@
 #include <Foundation/NSUserDefaults.h>
 #include <Foundation/NSAutoreleasePool.h>
 #include <Foundation/NSDebug.h>
+#include <Foundation/NSException.h>
 #include <AppKit/DPSOperators.h>
 #include <AppKit/NSApplication.h>
 #include <AppKit/NSCursor.h>
@@ -57,6 +58,10 @@
 
 #define	ROOT generic.appRootWindow
 
+
+static BOOL handlesWindowDecorations = YES;
+
+
 /*
  * Name for application root window.
  */
@@ -189,6 +194,11 @@ typedef struct {
 /* The atom */
 #define _XA_MOTIF_WM_HINTS "_MOTIF_WM_HINTS"
 
+
+Pixmap
+xgps_cursor_mask(Display *xdpy, Drawable draw, const char *data,
+                  int w, int h, int colors);
+
 /* Now the code */
 
 /* Set the style `styleMask' for the XWindow `window' using motif
@@ -232,7 +242,8 @@ static void setWindowHintsForStyle (Display *dpy, Window window,
   hints->functions = 0;
 
   /* Now add to the hints from the styleMask */
-  if (styleMask == NSBorderlessWindowMask)
+  if (styleMask == NSBorderlessWindowMask
+      || !handlesWindowDecorations)
     {
       hints->flags |= MWM_HINTS_DECORATIONS;
       hints->flags |= MWM_HINTS_FUNCTIONS;
@@ -321,6 +332,12 @@ static void setWindowHintsForStyle (Display *dpy, Window window,
 
 @implementation XGServer (WindowOps)
 
+-(BOOL) handlesWindowDecorations
+{
+  return handlesWindowDecorations;
+}
+
+
 /*
  * Where a window has been reparented by the wm, we use this method to
  * locate the window given knowledge of its border window.
@@ -361,14 +378,9 @@ static void setWindowHintsForStyle (Display *dpy, Window window,
 {
   gswindow_device_t	*win = (gswindow_device_t*)window;
   NSRect	x;
-  float	t, b, l, r;
 
-  [self styleoffsets: &l : &r : &t : &b : win->win_attrs.window_style];
-
-  x.size.width = o.size.width - l - r;
-  x.size.height = o.size.height - t - b;
-  x.origin.x = o.origin.x + l;
-  x.origin.y = o.origin.y + o.size.height - t;
+  x = o;
+  x.origin.y = x.origin.y + o.size.height;
   x.origin.y = DisplayHeight(dpy, win->screen) - x.origin.y;
 NSDebugLLog(@"Frame", @"O2X %d, %@, %@", win->number,
   NSStringFromRect(o), NSStringFromRect(x));
@@ -387,11 +399,11 @@ NSDebugLLog(@"Frame", @"O2X %d, %@, %@", win->number,
 
   [self styleoffsets: &l : &r : &t : &b : win->win_attrs.window_style];
 
-  x.size.width = o.size.width - l - r;
-  x.size.height = o.size.height - t - b;
-  x.origin.x = o.origin.x;
+  x.size.width = o.size.width;
+  x.size.height = o.size.height;
+  x.origin.x = o.origin.x - l;
   x.origin.y = o.origin.y + o.size.height;
-  x.origin.y = DisplayHeight(dpy, win->screen) - x.origin.y;
+  x.origin.y = DisplayHeight(dpy, win->screen) - x.origin.y - t;
 NSDebugLLog(@"Frame", @"O2H %d, %@, %@", win->number,
   NSStringFromRect(o), NSStringFromRect(x));
   return x;
@@ -423,14 +435,10 @@ NSDebugLLog(@"Frame", @"O2H %d, %@, %@", win->number,
 {
   gswindow_device_t	*win = (gswindow_device_t*)window;
   NSRect	o;
-  float	t, b, l, r;
 
-  [self styleoffsets: &l : &r : &t : &b : win->win_attrs.window_style];
-  o.size.width = x.size.width + l + r;
-  o.size.height = x.size.height + t + b;
-  o.origin.x = x.origin.x - l;
-  o.origin.y = DisplayHeight(dpy, win->screen) - x.origin.y;
-  o.origin.y = o.origin.y - o.size.height + t;
+  o = x;
+  o.origin.y = DisplayHeight(dpy, win->screen) - o.origin.y;
+  o.origin.y = o.origin.y - o.size.height;
 NSDebugLLog(@"Frame", @"X2O %d, %@, %@", win->number,
   NSStringFromRect(x), NSStringFromRect(o));
   return o;
@@ -744,6 +752,10 @@ NSDebugLLog(@"Frame", @"X2O %d, %@, %@", win->number,
    * Now check user defaults.
    */
   defs = [NSUserDefaults standardUserDefaults];
+
+  if ([defs objectForKey: @"GSX11HandlesWindowDecorations"])
+    handlesWindowDecorations = [defs boolForKey: @"GSX11HandlesWindowDecorations"];
+
   generic.flags.useWindowMakerIcons = NO;
   if ((generic.wm & XGWM_WINDOWMAKER) != 0)
     {
@@ -935,7 +947,20 @@ NSDebugLLog(@"Frame", @"X2O %d, %@, %@", win->number,
   root = [self _rootWindowForScreen: screen];
   context = [self xrContextForScreen: screen];
 
-  frame = [NSWindow contentRectForFrameRect: frame styleMask: style];
+  /* Create the window structure and set the style early so we can use it to
+  convert frames. */
+  window = objc_malloc(sizeof(gswindow_device_t));
+  memset(window, '\0', sizeof(gswindow_device_t));
+  window->display = dpy;
+  window->screen = screen;
+
+  window->win_attrs.flags |= GSWindowStyleAttr;
+  if (handlesWindowDecorations)
+    window->win_attrs.window_style = style;
+  else
+    window->win_attrs.window_style = style & (NSIconWindowMask | NSMiniWindowMask);
+
+  frame = [self _OSFrameToXFrame: frame for: window];
 
   /* We're not allowed to create a zero rect window */
   if (NSWidth(frame) <= 0 || NSHeight(frame) <= 0)
@@ -943,13 +968,6 @@ NSDebugLLog(@"Frame", @"X2O %d, %@, %@", win->number,
       frame.size.width = 2;
       frame.size.height = 2;
     }
-  /* Translate to X coordinates */
-  frame.origin.y = DisplayHeight(dpy, screen) - NSMaxY(frame);
-
-  window = objc_malloc(sizeof(gswindow_device_t));
-  memset(window, '\0', sizeof(gswindow_device_t));
-  window->display = dpy;
-  window->screen = screen;
   window->xframe = frame;
   window->type = type;
   window->root = root->ident;
@@ -966,7 +984,7 @@ NSDebugLLog(@"Frame", @"X2O %d, %@, %@", win->number,
 				context->depth,
 				CopyFromParent,
 				context->visual,
-				(CWColormap | CWBackPixel|CWBorderPixel),
+				(CWColormap | CWBackPixel | CWBorderPixel),
 				&window->xwn_attrs);
 
   /*
@@ -1002,15 +1020,13 @@ NSDebugLLog(@"Frame", @"X2O %d, %@, %@", win->number,
    * Initial attributes for any GNUstep window tell Window Maker not to
    * create an app icon for us.
    */
-  window->win_attrs.flags = GSExtraFlagsAttr;
-  window->win_attrs.extra_flags = GSNoApplicationIconFlag;
+  window->win_attrs.flags |= GSExtraFlagsAttr;
+  window->win_attrs.extra_flags |= GSNoApplicationIconFlag;
 
   /*
    * Prepare size/position hints, but don't set them now - ordering
    * the window in should automatically do it.
    */
-  window->win_attrs.flags |= GSWindowStyleAttr;
-  window->win_attrs.window_style = style;
   frame = [self _XFrameToOSFrame: window->xframe for: window];
   frame = [self _OSFrameToXHints: frame for: window];
   window->siz_hints.x = NSMinX(frame);
@@ -1125,6 +1141,17 @@ NSDebugLLog(@"Frame", @"X2O %d, %@, %@", win->number,
 - (void) styleoffsets: (float *) l : (float *) r : (float *) t : (float *) b 
 		     : (unsigned int) style
 {
+  if (!handlesWindowDecorations)
+    {
+      /*
+      If we don't handle decorations, all our windows are going to be
+      border- and decorationless. In that case, -gui won't call this method,
+      but we still use it internally.
+      */
+      *l = *r = *t = *b = 0.0;
+      return;
+    }
+
   /* First try to get the offset information that we have obtained from
      the WM. This will only work if the application has already created
      a window that has been reparented by the WM. Otherwise we have to
@@ -1212,6 +1239,8 @@ NSDebugLLog(@"Frame", @"X2O %d, %@, %@", win->number,
 {
   gswindow_device_t	*window;
 
+  NSAssert(handlesWindowDecorations, @"-stylewindow:: called when handlesWindowDecorations==NO");
+
   window = WINDOW_WITH_TAG(win);
   if (!window)
     return;
@@ -1385,11 +1414,7 @@ NSDebugLLog(@"Frame", @"X2O %d, %@, %@", win->number,
 */
 - (void) windowdevice: (int)win
 {
-  int      x, y;
   unsigned width, height;
-  unsigned old_width;
-  unsigned old_height;
-  XWindowAttributes winattr;
   gswindow_device_t *window;
   NSGraphicsContext *ctxt;
 
@@ -1404,43 +1429,11 @@ NSDebugLLog(@"Frame", @"X2O %d, %@, %@", win->number,
   if (!window->ident)
     return;
 
-  old_width = NSWidth(window->xframe);
-  old_height = NSHeight(window->xframe);
+  width = NSWidth(window->xframe);
+  height = NSHeight(window->xframe);
 
-  XFlush (dpy);
-
-  XGetGeometry(dpy, window->ident, &window->root,
-	       &x, &y, &width, &height,
-	       &window->border, &window->depth);
-  /* hack:
-   * wait until a resize of window is finished (especially for NSMenu)
-   * is there any way to wait until X finished it's stuff?
-   * XSync(), XFlush() doesn't do the job!
-   */
-  if (height != (unsigned int)(window->siz_hints.height))
-    {
-#if HAVE_USLEEP
-      usleep(1);
-#else
-      for (x=0; x<10000; x++)
-	;
-#endif
-      XGetGeometry(dpy, window->ident, &window->root,
-		   &x, &y, &width, &height,
-		   &window->border, &window->depth);
-    }
-
-  window->xframe.size.width = width;
-  window->xframe.size.height = height;
-
-  XGetWindowAttributes(dpy, window->ident, &winattr);
-  window->map_state = winattr.map_state;
-  
-  NSDebugLLog (@"NSWindow", @"window geom device ((%f, %f), (%f, %f))",
-	        window->xframe.origin.x,  window->xframe.origin.y, 
-	        window->xframe.size.width,  window->xframe.size.height);
-  
-  if (window->buffer && (old_width != width || old_height != height)
+  if (window->buffer
+      && (window->buffer_width != width || window->buffer_height != height)
       && (window->gdriverProtocol & GDriverHandlesBacking) == 0)
     {
       [isa waitAllContexts];
@@ -1451,6 +1444,9 @@ NSDebugLLog(@"Frame", @"X2O %d, %@, %@", win->number,
       window->alpha_buffer = 0;
     }
 
+  window->buffer_width = width;
+  window->buffer_height = height;
+
   if (window->buffer == 0)
     {
       [self _createBuffer: window];
@@ -1462,6 +1458,85 @@ NSDebugLLog(@"Frame", @"X2O %d, %@, %@", win->number,
   DPSinitclip(ctxt);
 }
 
+/*
+Build a Pixmap of our icon so the windowmaker dock will remember our
+icon when we quit.
+
+ICCCM really only allows 1-bit pixmaps for IconPixmapHint, but this code is
+only used if windowmaker is the window manager, and windowmaker can handle
+real color pixmaps.
+*/
+static Pixmap xIconPixmap;
+static Pixmap xIconMask;
+static BOOL didCreatePixmaps;
+
+-(int) _createAppIconPixmaps
+{
+  NSImage *image;
+  NSBitmapImageRep *rep;
+  int i, j, w, h, samples, screen;
+  unsigned char *data;
+  XColor pixelColor;
+  GC pixgc;
+  RColor pixelRColor;
+  RContext *rcontext;
+
+  NSAssert(!didCreatePixmaps, @"called _createAppIconPixmap twice");
+  
+  didCreatePixmaps = YES;
+  
+  image = [NSApp applicationIconImage];
+  rep = (NSBitmapImageRep *)[image bestRepresentationForDevice:nil];
+  
+  if (![rep isKindOfClass: [NSBitmapImageRep class]])
+    return 0;
+
+  if ([rep bitsPerSample] != 8
+      || (![[rep colorSpaceName] isEqual: NSDeviceRGBColorSpace]
+	  && ![[rep colorSpaceName] isEqual: NSCalibratedRGBColorSpace])
+      || [rep isPlanar])
+    return 0;
+  
+  data = [rep bitmapData];
+  screen = [[[self screenList] objectAtIndex: 0] intValue];
+  xIconPixmap = XCreatePixmap(dpy,
+                      [self xDisplayRootWindowForScreen:screen],
+                      [rep pixelsWide], [rep pixelsHigh],
+                      DefaultDepth(dpy, screen));
+  pixgc = XCreateGC(dpy, xIconPixmap, 0, NULL);
+
+  h = [rep pixelsHigh];
+  w = [rep pixelsWide];
+  samples = [rep samplesPerPixel];
+  rcontext = [self xrContextForScreen: screen];
+
+  for (i = 0; i < h; i++)
+    {
+      unsigned char *d = data;
+      for (j = 0; j < w; j++)
+	{
+	  pixelRColor.red = d[0];
+	  pixelRColor.green = d[1];
+	  pixelRColor.blue = d[2];
+
+	  RGetClosestXColor(rcontext, &pixelRColor, &pixelColor);
+	  XSetForeground(dpy, pixgc, pixelColor. pixel);
+	  XDrawPoint(dpy, xIconPixmap, pixgc, j, i);
+	  d += samples;
+	}
+      data += [rep bytesPerRow];
+    }
+  
+  XFreeGC(dpy, pixgc);
+ 
+  xIconMask = xgps_cursor_mask(dpy, ROOT, [rep bitmapData],
+			       [rep pixelsWide],
+			       [rep pixelsHigh],
+			       [rep samplesPerPixel]);
+
+  return 1;
+}   
+
 - (void) orderwindow: (int)op : (int)otherWin : (int)winNum
 {
   gswindow_device_t	*window;
@@ -1480,7 +1555,7 @@ NSDebugLLog(@"Frame", @"X2O %d, %@, %@", win->number,
       /*
        * Some window managers ignore any hints and properties until the
        * window is actually mapped, so we need to set them all up
-       * immediately bofore mapping the window ...
+       * immediately before mapping the window ...
        */
 
       setNormalHints(dpy, window);
@@ -1494,12 +1569,29 @@ NSDebugLLog(@"Frame", @"X2O %d, %@, %@", win->number,
       if ((window->win_attrs.window_style & NSIconWindowMask) != 0
 	&& generic.flags.useWindowMakerIcons == 1)
 	{
-	  XWMHints		gen_hints;
+	  XWMHints gen_hints;
 
 	  gen_hints.flags = WindowGroupHint | StateHint | IconWindowHint;
 	  gen_hints.initial_state = WithdrawnState;
 	  gen_hints.window_group = ROOT;
 	  gen_hints.icon_window = window->ident;
+
+	  if (!didCreatePixmaps)
+	    {
+	      [self _createAppIconPixmaps];
+	    }  
+
+	  if (xIconPixmap)
+	    {
+	      gen_hints.flags |= IconPixmapHint;
+	      gen_hints.icon_pixmap = xIconPixmap;
+	    }
+	  if (xIconMask);
+	    {
+	      gen_hints.flags |= IconMaskHint;
+	      gen_hints.icon_mask = xIconMask;
+	    }
+
 	  XSetWMHints(dpy, ROOT, &gen_hints);
 	}
 
@@ -1720,20 +1812,13 @@ NSDebugLLog(@"Frame", @"X2O %d, %@, %@", win->number,
 
 - (void) placewindow: (NSRect)rect : (int)win
 {
-  NSAutoreleasePool	*arp;
-  NSRect		xVal;
-  NSRect		last;
-  NSEvent		*event;
-  NSDate		*limit;
-  NSMutableArray	*tmpQueue;
-  unsigned		pos;
-  float			xdiff;
-  float			ydiff;
-  gswindow_device_t	*window;
-  NSWindow              *nswin;
-  NSRect		frame;
-  BOOL			resize = NO;
-  BOOL			move = NO;
+  NSEvent *e;
+  NSRect xVal;
+  gswindow_device_t *window;
+  NSWindow *nswin;
+  NSRect frame;
+  BOOL resize = NO;
+  BOOL move = NO;
 
   window = WINDOW_WITH_TAG(win);
   if (win == 0 || window == NULL)
@@ -1757,8 +1842,6 @@ NSDebugLLog(@"Frame", @"X2O %d, %@, %@", win->number,
     {
       move = YES;
     }
-  xdiff = rect.origin.x - frame.origin.x;
-  ydiff = rect.origin.y - frame.origin.y;
 
   xVal = [self _OSFrameToXHints: rect for: window];
   window->siz_hints.width = (int)xVal.size.width;
@@ -1767,8 +1850,6 @@ NSDebugLLog(@"Frame", @"X2O %d, %@, %@", win->number,
   window->siz_hints.y = (int)xVal.origin.y;
   xVal = [self _OSFrameToXFrame: rect for: window];
 
-  last = window->xframe;
-
   NSDebugLLog(@"Moving", @"Place %d - o:%@, x:%@", window->number,
     NSStringFromRect(rect), NSStringFromRect(xVal));
   XMoveResizeWindow (dpy, window->ident,
@@ -1776,124 +1857,42 @@ NSDebugLLog(@"Frame", @"X2O %d, %@, %@", win->number,
     window->siz_hints.width, window->siz_hints.height);
   setNormalHints(dpy, window);
 
-  /*
-   * Now massage all the events currently in the queue to make sure
-   * mouse locations in our window are adjusted as necessary.
-   */
-  arp = [NSAutoreleasePool new];
-  limit = [NSDate distantPast];	/* Don't wait for new events.	*/
-  tmpQueue = [NSMutableArray arrayWithCapacity: 8];
-  for (;;)
+  /* Update xframe right away. We optimistically assume that we'll get the
+  frame we asked for. If we're right, -gui can update/redraw right away,
+  and we don't have to do anything when the ConfigureNotify arrives later.
+  If we're wrong, the ConfigureNotify will have the exact coordinates, and
+  at that point, we'll send new GSAppKitWindow* events to -gui. */
+  window->xframe = xVal;
+
+  if (resize == YES)
     {
-      NSEventType	type;
-
-      event = DPSGetEvent(self, NSAnyEventMask, limit,
-	NSEventTrackingRunLoopMode);
-      if (event == nil)
-	break;
-      type = [event type];
-      if (type == NSAppKitDefined && [event windowNumber] == win)
-	{
-	  GSAppKitSubtype	sub = [event subtype];
-
-	  /*
-	   * Window movement or resize events for the window we are
-	   * watching are posted immediately, so they can take effect
-	   * before the placewindow returns.
-	   */
-	  if (sub == GSAppKitWindowMoved || sub == GSAppKitWindowResized)
-	    {
-	      [nswin sendEvent: event];
-	    }
-	  else
-	    {
-	      [tmpQueue addObject: event];
-	    }
-	}
-      else if (type != NSPeriodic && type != NSLeftMouseDragged
-	&& type != NSOtherMouseDragged && type != NSRightMouseDragged
-	&& type != NSMouseMoved)
-	{
-	  /*
-	   * Save any events that arrive before our window is moved - excepting
-	   * periodic events (which we can assume will be outdated) and mouse
-	   * movement events (which might flood us).
-	   */
-	  [tmpQueue addObject: event];
-	}
-      if (NSEqualRects(xVal, window->xframe) == YES ||
-	NSEqualRects(rect, [nswin frame]) == YES)
-	{
-	  break;
-	}
-      if (NSEqualRects(last, window->xframe) == NO)
-	{
-	  NSDebugLLog(@"Moving", @"From: %@\nWant %@\nGot %@",
-	    NSStringFromRect(last),
-	    NSStringFromRect(xVal),
-	    NSStringFromRect(window->xframe));
-	  last = window->xframe;
-	}
+      NSDebugLLog(@"Moving", @"Fake size %d - %@", window->number,
+	NSStringFromSize(rect.size));
+      e = [NSEvent otherEventWithType: NSAppKitDefined
+			     location: rect.origin
+			modifierFlags: 0
+			    timestamp: 0
+			 windowNumber: win
+			      context: GSCurrentContext()
+			      subtype: GSAppKitWindowResized
+				data1: rect.size.width
+				data2: rect.size.height];
+      [nswin sendEvent: e];
     }
-  /*
-   * If we got any events while waiting for the window movement, we
-   * may need to adjust their locations to match the new window position.
-   */
-  pos = [tmpQueue count];
-  while (pos-- > 0) 
+  else if (move == YES)
     {
-      event = [tmpQueue objectAtIndex: pos];
-      if ([event windowNumber] == win)
-	{
-	  NSPoint	loc = [event locationInWindow];
-
-	  loc.x -= xdiff;
-	  loc.y -= ydiff;
-	  [event _patchLocation: loc];
-	}
-      DPSPostEvent(self, event, YES);
-    }
-  RELEASE(arp);
-
-  /*
-   * Failsafe - if X hasn't told us it has moved/resized the window, we
-   * fake the notification and post them immediately, so they can take
-   * effect before the placewindow returns.
-   */
-  if (NSEqualRects([nswin frame], rect) == NO)
-    {
-      NSEvent	*e;
-
-      if (resize == YES)
-	{
-	  NSDebugLLog(@"Moving", @"Fake size %d - %@", window->number,
-	    NSStringFromSize(rect.size));
-	  e = [NSEvent otherEventWithType: NSAppKitDefined
-				 location: rect.origin
-			    modifierFlags: 0
-				timestamp: 0
-			     windowNumber: win
-		                  context: GSCurrentContext()
-				  subtype: GSAppKitWindowResized
-				    data1: rect.size.width
-				    data2: rect.size.height];
-	  [nswin sendEvent: e];
-	}
-      if (move == YES)
-	{
-	  NSDebugLLog(@"Moving", @"Fake move %d - %@", window->number,
-	    NSStringFromPoint(rect.origin));
-	  e = [NSEvent otherEventWithType: NSAppKitDefined
-				 location: NSZeroPoint
-			    modifierFlags: 0
-				timestamp: 0
-			     windowNumber: win
-		                  context: GSCurrentContext()
-				  subtype: GSAppKitWindowMoved
-				    data1: rect.origin.x
-				    data2: rect.origin.y];
-	  [nswin sendEvent: e];
-	}
+      NSDebugLLog(@"Moving", @"Fake move %d - %@", window->number,
+	NSStringFromPoint(rect.origin));
+      e = [NSEvent otherEventWithType: NSAppKitDefined
+			     location: NSZeroPoint
+			modifierFlags: 0
+			    timestamp: 0
+			 windowNumber: win
+			      context: GSCurrentContext()
+			      subtype: GSAppKitWindowMoved
+				data1: rect.origin.x
+				data2: rect.origin.y];
+      [nswin sendEvent: e];
     }
 }
 
@@ -2369,6 +2368,9 @@ NSDebugLLog(@"Frame", @"X2O %d, %@, %@", win->number,
  */
 - (void) setinputstate: (int)st : (int)win
 {
+  if (!handlesWindowDecorations)
+    return;
+
   NSDebugLLog(@"XGTrace", @"DPSsetinputstate: %d : %d", st, win);
   if ((generic.wm & XGWM_WINDOWMAKER) != 0)
     {
@@ -2658,6 +2660,33 @@ xgps_cursor_image(Display *xdpy, Drawable draw, const unsigned char *data,
     case GSIBeamCursor:
       cursor->c = XCreateFontCursor(dpy, XC_xterm);
       break;
+    case GSCrosshairCursor:
+      cursor->c = XCreateFontCursor(dpy, XC_crosshair);
+      break;
+    case GSDisappearingItemCursor:
+      cursor->c = XCreateFontCursor(dpy, XC_shuttle);
+      break;
+    case GSPointingHandCursor:
+      cursor->c = XCreateFontCursor(dpy, XC_hand1);
+      break;
+    case GSResizeDownCursor:
+      cursor->c = XCreateFontCursor(dpy, XC_bottom_side);
+      break;
+    case GSResizeLeftCursor:
+      cursor->c = XCreateFontCursor(dpy, XC_left_side);
+      break;
+    case GSResizeLeftRightCursor:
+      cursor->c = XCreateFontCursor(dpy, XC_sb_h_double_arrow);
+      break;
+    case GSResizeRightCursor:
+      cursor->c = XCreateFontCursor(dpy, XC_right_side);
+      break;
+    case GSResizeUpCursor:
+      cursor->c = XCreateFontCursor(dpy, XC_top_side);
+      break;
+    case GSResizeUpDownCursor:
+      cursor->c = XCreateFontCursor(dpy, XC_sb_v_double_arrow);
+      break;
     default:
       cursor->c = XCreateFontCursor(dpy, XC_left_ptr);     
       break;
