@@ -151,7 +151,7 @@ static	Region	emptyRegion;
   XGServer *srv;
   gswindow_device_t *gs_win;
 
-  gs_win = (gswindow_device_t *)windevice = device;
+  gs_win = windevice = device;
   draw = GET_XDRAWABLE(gs_win);
   [self setGraphicContext: gs_win->gc];
   alpha_buffer = 0;
@@ -1175,127 +1175,6 @@ static	Region	emptyRegion;
 /* ----------------------------------------------------------------------- */
 /* Text operations */
 /* ----------------------------------------------------------------------- */
-typedef enum {
-  show_delta, show_array_x, show_array_y, show_array_xy
-} show_array_t;
-
-/* Omnibus show string routine that combines that characteristics of
-   ashow, awidthshow, widthshow, xshow, xyshow, and yshow */
-- (void) _showString: (const char *)s
-	    xCharAdj: (float)cx
-	    yCharAdj: (float)cy
-		char: (char)c
-	    adjArray: (const float *)arr
-	     arrType: (show_array_t)type
-	  isRelative: (BOOL)relative;
-{
-  int i;
-  int len;
-  int width;
-  NSSize scale;
-  NSPoint point = [path currentPoint];
-
-  if (font == nil)
-    {
-      NSLog(@"DPS (xgps): no font set\n");
-      return;
-    }
-
-  COPY_GC_ON_CHANGE; 
-  if (draw == 0)
-    {
-      DPS_WARN(DPSinvalidid, @"No Drawable defined for show");
-      return;
-    }
-
-  if ((cstate & COLOR_FILL) == 0)
-    [self setColor: &fillColor state: COLOR_FILL];
-
-  /* Use only delta transformations (no offset) */
-  len = strlen(s);
-  scale = [ctm sizeInMatrixSpace: NSMakeSize(1,1)];
-  for (i = 0; i < len; i++)
-    {
-      NSPoint	delta;
-      XPoint	xp;
-
-      // FIXME: We should put this line before the loop
-      // and do all computation in display space.
-      xp = XGWindowPointToX(self, point);
-      width = [(XGFontInfo *)font widthOf: s+i lenght: 1];
-      // Hack: Only draw when alpha is not zero
-      if (drawingAlpha == NO || fillColor.field[AINDEX] != 0.0)
-	[(XGFontInfo *)font draw: s+i lenght: 1 
-		   onDisplay: XDPY drawable: draw
-		   with: xgcntxt at: xp];
-
-      if (drawingAlpha)
-	{
-	  NSAssert(alpha_buffer, NSInternalInconsistencyException);
-	  
-	  [self setAlphaColor: fillColor.field[AINDEX]];
-	  [(XGFontInfo *)font draw: s+i lenght: 1 
-		     onDisplay: XDPY drawable: alpha_buffer
-		     with: agcntxt at: xp];
-	}
-      /* Note we update the current point according to the current 
-	 transformation scaling, although the text isn't currently
-	 scaled (FIXME). */
-      if (type == show_array_xy)
-	{
-	  delta.x = arr[2*i]; delta.y = arr[2*i+1];
-	}
-      else if (type == show_array_x)
-	{
-	  delta.x = arr[i]; delta.y = 0;
-	}
-      else if (type == show_array_y)
-	{
-	  delta.x = 0; delta.y = arr[i];
-	}
-      else
-	{
-	  delta.x = arr[0]; delta.y = arr[1];
-	}
-      delta = [ctm deltaPointInMatrixSpace: delta];
-      if (relative == YES)
-	{
-	  delta.x += width * scale.width;
-	  delta.y += [font ascender] * scale.height;
-	}
-      if (c && *(s+i) == c)
-	{
-	  NSPoint cdelta;
-	  cdelta.x = cx; cdelta.y = cy;
-	  cdelta = [ctm deltaPointInMatrixSpace: cdelta];
-	  delta.x += cdelta.x; delta.y += cdelta.y;
-	}
-      point.x += delta.x;
-      if (type != show_delta)
-	point.y += delta.y;
-    }
-  // FIXME: Should we set the current point now?
-}
-
-- (void)DPSashow: (float)x : (float)y : (const char *)s 
-{
-  float arr[2];
-
-  arr[0] = x; arr[1] = y;
-  [self _showString: s
-    xCharAdj: 0 yCharAdj: 0 char: 0 adjArray: arr arrType: show_delta
-    isRelative: YES];
-}
-
-- (void)DPSawidthshow: (float)cx : (float)cy : (int)c : (float)ax : (float)ay : (const char *)s 
-{
-  float arr[2];
-
-  arr[0] = ax; arr[1] = ay;
-  [self _showString: s
-    xCharAdj: cx yCharAdj: cy char: c adjArray: arr arrType: show_delta
-    isRelative: YES];
-}
 
 - (void)DPSshow: (const char *)s 
 {
@@ -1346,7 +1225,6 @@ typedef enum {
   [path relativeMoveToPoint: NSMakePoint(width * scale.width, 0)];
 }
 
-
 - (void) GSShowGlyphs: (const NSGlyph *)glyphs : (size_t) length
 {
   int width;
@@ -1394,37 +1272,6 @@ typedef enum {
   [path relativeMoveToPoint: NSMakePoint(width * scale.width, 0)];
 }
 
-- (void)DPSwidthshow: (float)x : (float)y : (int)c : (const char *)s 
-{
-  float arr[2];
-
-  arr[0] = 0; arr[1] = 0;
-  [self _showString: s
-    xCharAdj: x yCharAdj: y char: c adjArray: arr arrType: show_delta
-    isRelative: YES];
-}
-
-- (void)DPSxshow: (const char *)s : (const float *)numarray : (int)size 
-{
-  [self _showString: s
-    xCharAdj: 0 yCharAdj: 0 char: 0 adjArray: numarray arrType: show_array_x
-    isRelative: NO];
-}
-
-- (void)DPSxyshow: (const char *)s : (const float *)numarray : (int)size 
-{
-  [self _showString: s
-    xCharAdj: 0 yCharAdj: 0 char: 0 adjArray: numarray arrType: show_array_xy
-    isRelative: NO];
-}
-
-- (void)DPSyshow: (const char *)s : (const float *)numarray : (int)size 
-{
-  [self _showString: s
-    xCharAdj: 0 yCharAdj: 0 char: 0 adjArray: numarray arrType: show_array_y
-    isRelative: NO];
-}
-
 - (void) GSSetFont: (GSFontInfo *)newFont
 {
   if (font == newFont)
@@ -1868,6 +1715,7 @@ typedef enum {
   gswindow_device_t *source_win;
   NSMutableDictionary *dict;
   NSData *data;
+  NSAffineTransform *matrix;
 
   source_win = (gswindow_device_t *)windevice;
   if (!source_win)
@@ -1892,22 +1740,41 @@ typedef enum {
   srect = XGIntersectionRect (srect, accessibleRectForWindow (source_win));
   ssize.width = srect.width;
   ssize.height = srect.height;
-  [dict setObject: [NSValue valueWithSize: ssize] forKey: @"ImageSize"];
+  [dict setObject: [NSValue valueWithSize: ssize] forKey: @"Size"];
+
+  [dict setObject: NSDeviceRGBColorSpace forKey: @"ColorSpace"];
+  [dict setObject: [NSNumber numberWithUnsignedInt: 8] forKey: @"BitsPerSample"];
+  [dict setObject: [NSNumber numberWithUnsignedInt: source_win->depth]
+	forKey: @"Depth"];
+  [self _alphaBuffer: source_win];
+  if (alpha_buffer)
+    {
+      [dict setObject: [NSNumber numberWithUnsignedInt: 4] 
+	    forKey: @"SamplesPerPixel"];
+      [dict setObject: [NSNumber numberWithUnsignedInt: 1]
+	    forKey: @"HasAlpha"];
+    }
+  else
+    {
+      [dict setObject: [NSNumber numberWithUnsignedInt: 3] 
+	    forKey: @"SamplesPerPixel"];
+      [dict setObject: [NSNumber numberWithUnsignedInt: 0]
+	    forKey: @"HasAlpha"];
+    }
+  matrix = [ctm copy];
+  [matrix translateXBy: -srect.x - offset.x yBy: srect.y + srect.height - offset.y];
+  [dict setObject: matrix forKey: @"Matrix"];
+  DESTROY(matrix);
 
   if (XGIsEmptyRect(srect))
     return dict;
 
-  [dict setObject: [NSNumber numberWithUnsignedInt: source_win->depth]
-	forKey: @"ImageDepth"];
-
   // --- get source XImage ----------------------------------------
   
   if (draw == source_win->ident && source_win->visibility < 0)
     {
-      /* Non-backingstore window isn't visible, so just make up the image */
-      NSLog(@"Focused view window not readable");
-      source_im = RCreateXImage(context, source_win->depth, 
-			      XGWidth(srect), XGHeight(srect));
+      /* Non-backingstore window isn't visible, so we can't read it. */
+      return nil;
     }
   else
     {
@@ -1918,36 +1785,24 @@ typedef enum {
   if (source_im->image == 0)
     {
       // Should not happen, 
-      DPS_ERROR (DPSinvalidaccess, @"unable to fetch source image");
-      return dict;
+      return nil;
     }
   
-  [self _alphaBuffer: source_win];
   if (alpha_buffer)
     {
       source_alpha = RGetXImage((RContext *)context, alpha_buffer,
                               XGMinX(srect), XGMinY(srect), 
                               XGWidth(srect), XGHeight(srect));
-      [dict setObject: [NSNumber numberWithUnsignedInt: 4] 
-	    forKey: @"ImageSPP"];
-      [dict setObject: [NSNumber numberWithUnsignedInt: 1]
-	    forKey: @"ImageAlpha"];
     }
   else
     {
       source_alpha = NULL;
-      [dict setObject: [NSNumber numberWithUnsignedInt: 3] 
-	    forKey: @"ImageSPP"];
-      [dict setObject: [NSNumber numberWithUnsignedInt: 0]
-	    forKey: @"ImageAlpha"];
     }
 
   data = _pixmap_read_alpha(context, source_im, source_alpha, srect,
 			    drawMechanism);
-  [dict setObject: data forKey: @"ImageData"];
+  [dict setObject: data forKey: @"Data"];
   /* Pixmap routine always returns image in same format (FIXME?).  */
-  [dict setObject: NSDeviceRGBColorSpace forKey: @"ImageColorSpace"];
-  [dict setObject: [NSNumber numberWithUnsignedInt: 8] forKey: @"ImageBPS"];
 
   // --- clean up ------------------------------------------------------
   
